div.autodoc-docstring {
    padding-left: 20px;
    margin-bottom: 30px;
    border-left: 5px solid rgba(230, 230, 230);
}

div.autodoc-members {
    padding-left: 20px;
    margin-bottom: 15px;
}

div.md-content {
    min-width: fit-content;
}

div.md-sidebar {
    min-width: fit-content;
}

.rst-content code {
    color: darkblue;
    font-family: "Roboto Slab", sans-serif;
    font-size: 14px;
    border-radius: 15px;
    padding: 4px 10px;
}

.doc {
    font-family: "Roboto Slab", sans-serif;
}

.doc-label {
    border-radius: 15px;
    padding: 2px 8px;
    font-weight: bold;
    color: darkgreen;
}

.doc-label-special {
    background-color: #3330E4;
    color: white;
}

.doc-label-private {
    background-color: #F637EC;
    color: white;
}

.doc-label-property {
    background-color: #FBB454;
    color: black;
}

.doc-label-read-only {
    background-color: #FAEA48;
    color: black;
}
